// *************************************************************************************
// SCICHART® Copyright SciChart Ltd. 2011-2021. All rights reserved.
//  
// Web: http://www.scichart.com
//   Support: support@scichart.com
//   Sales:   sales@scichart.com
// 
// SeriesVerticalSlicesExample.xaml.cs is part of the SCICHART® Examples. Permission is hereby granted
// to modify, create derivative works, distribute and publish any part of this source
// code whether for commercial, private or personal use. 
// 
// The SCICHART® examples are distributed in the hope that they will be useful, but
// without any warranty. It is provided "AS IS" without warranty of any kind, either
// expressed or implied. 
// *************************************************************************************

using SciChart.Charting.ChartModifiers;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.Visuals.Annotations;
using SciChart.Core.Utility.Mouse;
using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace SciChart_UsingVerticalSliceTooltips
{
    public class CTWVerticalSliceModifier : VerticalSliceModifier
    {
        #region Overrides of InspectSeriesModifierBase

        public override void OnModifierMouseMove(ModifierMouseArgs e)
        {
            base.OnModifierMouseMove(e);

            foreach (var renderSeries in ParentSurface.RenderableSeries)
            {
                var hitTestInfo = renderSeries.HitTestProvider.HitTest(e.MousePoint);
            }
        }

        #endregion
    }

    /// <summary>
    /// Interaction logic for SeriesVerticalSlicesExample.xaml
    /// </summary>
    public partial class SeriesVerticalSlicesExample : UserControl
    {
        // A drop in replacement for System.Random which is 3x faster: https://www.codeproject.com/Articles/9187/A-fast-equivalent-for-System-Random
        private readonly Random _random = new Random();

        public SeriesVerticalSlicesExample()
        {
            InitializeComponent();
            Loaded += OnLoaded;
        }

        private void OnLoaded(object sender, RoutedEventArgs routedEventArgs)
        {
            var dataSeries0 = new XyDataSeries<double>() { SeriesName = "Curve A", AcceptsUnsortedData = true};
            var dataSeries1 = new XyDataSeries<double>() { SeriesName = "Curve B", AcceptsUnsortedData = true};
            var dataSeries2 = new XyDataSeries<double>() { SeriesName = "Curve C", AcceptsUnsortedData = true};

            for(int i = 0; i < 3600; ++i)
            {
                double x = Math.Cos((i / 3600.0) * 2 * Math.PI);
                double y = Math.Sin((i / 3600.0) * 2 * Math.PI);

                dataSeries0.Append(x * 1.0, y * 1.0);
                dataSeries1.Append(x * 2.0, y * 2.0);
                dataSeries2.Append(x * 3.0, y * 3.0);
            }

            using (sciChart.SuspendUpdates())
            {
                sciChart.RenderableSeries[0].DataSeries = dataSeries0;
                sciChart.RenderableSeries[1].DataSeries = dataSeries1;
                sciChart.RenderableSeries[2].DataSeries = dataSeries2;
            }

            sciChart.ZoomExtents();
        }

        private void OnCreateSliceClick(object sender, RoutedEventArgs e)
        {
            MouseButtonEventHandler mouseClick = null;

            mouseClick = (s, arg) =>
            {
                MouseLeftButtonUp -= mouseClick;

                var mousePoint = arg.GetPosition((UIElement)sciChart.GridLinesPanel).X;

                var slice = new VerticalLineAnnotation()
                {
                    X1 = sciChart.XAxis.GetDataValue(mousePoint),
                    Style = (Style)Resources["sliceStyle"]
                };

                sliceModifier.VerticalLines.Add(slice);
            };

            MouseLeftButtonUp += mouseClick;
        }

        private void OnDeleteSelectedSliceClick(object sender, RoutedEventArgs e)
        {
            var selectedSlices = sliceModifier.VerticalLines.Where(annotation => annotation.IsSelected).ToList();

            foreach (var slice in selectedSlices)
            {
                sciChart.Annotations.Remove(slice);
            }
        }
    }
}
